package com.ruoyi.system.user.service.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import javax.validation.Validator;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.support.TransactionTemplate;

import com.github.pagehelper.PageHelper;
import com.ruoyi.common.constant.CacheConstants;
import com.ruoyi.common.constant.UserConstants;
import com.ruoyi.common.core.domain.entity.SysRole;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.core.validate.ValidGroup;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.bean.BeanValidators;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.system.dept.domain.SysDeptVO;
import com.ruoyi.system.dept.service.ISysDeptService;
import com.ruoyi.system.domain.SysPost;
import com.ruoyi.system.domain.SysUserPost;
import com.ruoyi.system.domain.SysUserRole;
import com.ruoyi.system.mapper.SysPostMapper;
import com.ruoyi.system.mapper.SysRoleMapper;
import com.ruoyi.system.mapper.SysUserPostMapper;
import com.ruoyi.system.mapper.SysUserRoleMapper;
import com.ruoyi.system.service.ISysConfigService;
import com.ruoyi.system.service.ISysRoleService;
import com.ruoyi.system.user.domain.SysUserQuery;
import com.ruoyi.system.user.domain.SysUserVO;
import com.ruoyi.system.user.domain.vo.SysUserNameVO;
import com.ruoyi.system.user.mapper.SysUserMapper;
import com.ruoyi.system.user.service.UserService;
import com.ruoyi.system.user.service.UserValidator;

/**
 * 用户 业务层处理
 *
 * @author ruoyi
 */
@Service
public class UserServiceImpl implements UserService
{
    private static final Logger log = LoggerFactory.getLogger(UserServiceImpl.class);

    @Autowired
    private SysUserMapper userMapper;

    @Autowired
    private SysRoleMapper roleMapper;

    @Autowired
    private SysPostMapper postMapper;

    @Autowired
    private SysUserRoleMapper userRoleMapper;

    @Autowired
    private SysUserPostMapper userPostMapper;

    @Autowired
    private ISysConfigService configService;

    @Autowired
    private ISysDeptService deptService;

    @Autowired
    protected Validator validator;

    @Autowired
    private ScheduledExecutorService scheduledExecutorService;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Autowired
    private UserValidator userValidator;

    @Autowired
    private ISysRoleService roleService;

    @Autowired
    private RedisCache redisCache;


    /**
     * 根据条件分页查询用户列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    @Override
    public List<SysUserVO> selectUserListScope(SysUserQuery user) {
        return userMapper.selectUserList(user);
    }

    /**
     * 根据条件分页查询已分配用户角色列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    @Override
    public List<SysUser> selectAllocatedList(SysUser user)
    {
        return userMapper.selectAllocatedList(user);
    }

    /**
     * 根据条件分页查询未分配用户角色列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    @Override
    public List<SysUser> selectUnallocatedList(SysUser user)
    {
        return userMapper.selectUnallocatedList(user);
    }

    /**
     * 通过用户名查询用户
     *
     * @param userName 用户名
     * @return 用户对象信息
     */
    @Override
    public SysUser selectUserByUserName(String userName)
    {
        return userMapper.selectUserByUserName(userName);
    }

    /**
     * 通过用户ID查询用户
     *
     * @param userId 用户ID
     * @return 用户对象信息
     */
    @Override
    public SysUser selectUserById(Long userId)
    {
        return userMapper.selectUserById(userId);
    }

    /**
     * 查询用户所属角色组
     *
     * @param userName 用户名
     * @return 结果
     */
    @Override
    public String selectUserRoleGroup(String userName)
    {
        List<SysRole> list = roleMapper.selectRolesByUserName(userName);
        if (CollectionUtils.isEmpty(list))
        {
            return StringUtils.EMPTY;
        }
        return list.stream().map(SysRole::getRoleName).collect(Collectors.joining(","));
    }

    /**
     * 查询用户所属岗位组
     *
     * @param userName 用户名
     * @return 结果
     */
    @Override
    public String selectUserPostGroup(String userName)
    {
        List<SysPost> list = postMapper.selectPostsByUserName(userName);
        if (CollectionUtils.isEmpty(list))
        {
            return StringUtils.EMPTY;
        }
        return list.stream().map(SysPost::getPostName).collect(Collectors.joining(","));
    }

    /**
     * 校验手机号码是否唯一
     *
     * @param user 用户信息
     * @return
     */
    @Override
    public boolean checkPhoneUnique(SysUser user)
    {
        Long userId = StringUtils.isNull(user.getUserId()) ? -1L : user.getUserId();
        SysUser info = userMapper.checkPhoneUnique(user.getPhonenumber());
        if (StringUtils.isNotNull(info) && info.getUserId().longValue() != userId.longValue())
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 校验email是否唯一
     *
     * @param user 用户信息
     * @return
     */
    @Override
    public boolean checkEmailUnique(SysUser user)
    {
        Long userId = StringUtils.isNull(user.getUserId()) ? -1L : user.getUserId();
        SysUser info = userMapper.checkEmailUnique(user.getEmail());
        if (StringUtils.isNotNull(info) && info.getUserId().longValue() != userId.longValue())
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 校验用户是否允许操作
     *
     * @param user 用户信息
     */
    @Override
    public void checkUserAllowed(SysUser user)
    {
        if (StringUtils.isNotNull(user.getUserId()) && user.isAdmin())
        {
            throw new ServiceException("不允许操作超级管理员用户");
        }
    }

    /**
     * 校验用户是否有数据权限
     *
     * @param userId 用户id
     */
    @Override
    public void checkUserDataScope(Long userId)
    {
        if (!SysUser.isAdmin(SecurityUtils.getUserId()))
        {
            SysUserQuery user = new SysUserQuery();
            user.setUserId(userId);
            List<SysUserVO> users = this.selectUserListScope(user);
            if (StringUtils.isEmpty(users))
            {
                throw new ServiceException("没有权限访问用户数据！");
            }
        }
    }

    /**
     * 新增保存用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public int insertUser(SysUser user)
    {
        // 校验
        userValidator.checkUserNoUnique(user.getUserId(), user.getUserNo());
        deptService.checkDeptDataScope(user.getDeptId());
        roleService.checkRoleDataScope(user.getRoleIds());
        ServiceUtil.assertion(StringUtils.isNotEmpty(user.getPhonenumber()) && !this.checkPhoneUnique(user),
                "新增用户'" + user.getUserName() + "'失败，手机号码已存在"
        );
        ServiceUtil.assertion(StringUtils.isNotEmpty(user.getEmail()) && !this.checkEmailUnique(user),
                "新增用户'" + user.getUserName() + "'失败，邮箱账号已存在"
        );

        // 填充数据
        user.setPassword(SecurityUtils.encryptPassword(user.getPassword()));
        // 操作数据库
        Integer result = transactionTemplate.execute(status -> {
            // 新增用户信息
            int rows = userMapper.insertUser(user);
            // 新增用户岗位关联
            insertUserPost(user);
            // 新增用户与角色管理
            insertUserRole(user);

            if (rows > 0) {
                this.clearCache();
            }

            return rows;
        });

        return result == null ? 0 : result;
    }

    void clearCache() {
        redisCache.deleteObject(CacheConstants.USER_NAME_LIST);
    }

    /**
     * 注册用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public boolean registerUser(SysUser user)
    {
        int rows = this.insertUser(user);

        if (rows > 0) {
            clearCache();
        }
        return rows > 0;
    }

    /**
     * 修改保存用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public int updateUser(SysUser user) {
        // 校验
        Long userId = user.getUserId();
        userValidator.checkUserNoUnique(user.getUserId(), user.getUserNo());
        this.checkUserAllowed(user);
        this.checkUserDataScope(user.getUserId());
        deptService.checkDeptDataScope(user.getDeptId());
        roleService.checkRoleDataScope(user.getRoleIds());
        ServiceUtil.assertion(StringUtils.isNotEmpty(user.getPhonenumber()) && !this.checkPhoneUnique(user),
                "修改用户'" + user.getUserName() + "'失败，手机号码已存在");
        ServiceUtil.assertion(StringUtils.isNotEmpty(user.getEmail()) && !this.checkEmailUnique(user),
                "修改用户'" + user.getUserName() + "'失败，邮箱账号已存在");

        Integer result = transactionTemplate.execute(status -> {
            // 删除用户与角色关联
            userRoleMapper.deleteUserRoleByUserId(userId);
            // 新增用户与角色管理
            insertUserRole(user);
            // 删除用户与岗位关联
            userPostMapper.deleteUserPostByUserId(userId);
            // 新增用户与岗位管理
            insertUserPost(user);
            int rows = userMapper.updateUser(user);

            if (rows > 0) {
                this.clearCache();
            }

            return rows;
        });

        return result == null ? 0 : result;
    }

    /**
     * 用户授权角色
     *
     * @param userId 用户ID
     * @param roleIds 角色组
     */
    @Override
    @Transactional
    public void insertUserAuth(Long userId, Long[] roleIds)
    {
        userRoleMapper.deleteUserRoleByUserId(userId);
        insertUserRole(userId, roleIds);
    }

    /**
     * 修改用户状态
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public int updateUserStatus(SysUser user)
    {
        int rows = userMapper.updateUser(user);

        if (rows > 0) {
            clearCache();
        }

        return rows;
    }

    /**
     * 修改用户基本信息
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public int updateUserProfile(SysUser user)
    {
        int rows = userMapper.updateUser(user);

        if (rows > 0) {
            clearCache();
        }

        return rows;
    }

    /**
     * 修改用户头像
     *
     * @param userName 用户名
     * @param avatar 头像地址
     * @return 结果
     */
    @Override
    public boolean updateUserAvatar(String userName, String avatar)
    {
        int rows = userMapper.updateUserAvatar(userName, avatar);

        if (rows > 0) {
            clearCache();
        }

        return rows > 0;
    }

    /**
     * 重置用户密码
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public int resetPwd(SysUser user)
    {
        return userMapper.updateUser(user);
    }

    /**
     * 重置用户密码
     *
     * @param userName 用户名
     * @param password 密码
     * @return 结果
     */
    @Override
    public int resetUserPwd(String userName, String password)
    {
        return userMapper.resetUserPwd(userName, password);
    }

    /**
     * 新增用户角色信息
     *
     * @param user 用户对象
     */
    public void insertUserRole(SysUser user)
    {
        this.insertUserRole(user.getUserId(), user.getRoleIds());
    }

    /**
     * 新增用户岗位信息
     *
     * @param user 用户对象
     */
    public void insertUserPost(SysUser user)
    {
        Long[] posts = user.getPostIds();
        if (StringUtils.isNotEmpty(posts))
        {
            // 新增用户与岗位管理
            List<SysUserPost> list = new ArrayList<SysUserPost>(posts.length);
            for (Long postId : posts)
            {
                SysUserPost up = new SysUserPost();
                up.setUserId(user.getUserId());
                up.setPostId(postId);
                list.add(up);
            }
            userPostMapper.batchUserPost(list);
        }
    }

    /**
     * 新增用户角色信息
     *
     * @param userId 用户ID
     * @param roleIds 角色组
     */
    public void insertUserRole(Long userId, Long[] roleIds)
    {
        if (userId == null) {
            return;
        }
        if (StringUtils.isNotEmpty(roleIds))
        {
            // 新增用户与角色管理
            List<SysUserRole> list = new ArrayList<SysUserRole>(roleIds.length);
            for (Long roleId : roleIds)
            {
                if (roleId == null) {
                    continue;
                }
                SysUserRole ur = new SysUserRole();
                ur.setUserId(userId);
                ur.setRoleId(roleId);
                list.add(ur);
            }
            if (CollectionUtils.isNotEmptyElement(list)) {
                userRoleMapper.batchUserRole(list);
            }
        }
    }

    /**
     * 通过用户ID删除用户
     *
     * @param userId 用户ID
     * @return 结果
     */
    @Override
    @Transactional
    public int deleteUserById(Long userId)
    {
        // 删除用户与角色关联
        userRoleMapper.deleteUserRoleByUserId(userId);
        // 删除用户与岗位表
        userPostMapper.deleteUserPostByUserId(userId);
        int rows = userMapper.deleteUserById(userId);

        if (rows > 0) {
            clearCache();
        }

        return rows;
    }

    /**
     * 批量删除用户信息
     *
     * @param userIds 需要删除的用户ID
     * @return 结果
     */
    @Override
    @Transactional
    public int deleteUserByIds(Long[] userIds)
    {
        for (Long userId : userIds)
        {
            checkUserAllowed(new SysUser(userId));
            checkUserDataScope(userId);
        }
        // 删除用户与角色关联
        userRoleMapper.deleteUserRole(userIds);
        // 删除用户与岗位关联
        userPostMapper.deleteUserPost(userIds);
        int rows = userMapper.deleteUserByIds(userIds);

        if (rows > 0) {
            clearCache();
        }

        return rows;
    }

    /**
     * 导入用户数据
     *
     * @param userList        用户数据列表
     * @param isUpdateSupport 是否更新支持，如果已存在，则进行更新数据
     * @param operaId         操作用户
     * @return 结果
     */
    @Override
    public int importUser(List<SysUserVO> userList, Boolean isUpdateSupport, Long operaId) {
        if (CollectionUtils.isEmptyElement(userList)) {
            throw new ServiceException("导入用户数据不能为空！");
        }

        SysUser operator = this.selectUserById(operaId);
        ServiceUtil.assertion(operator == null, "操作人不存在");

        // 开始异步导入
        scheduledExecutorService.schedule(() -> {

            // 查询部门列表
            List<SysDeptVO> deptList = deptService.selectDeptListByNames(CollectionUtils.map(userList, SysUserVO::getDeptName));
            SysDeptVO defaultDept = new SysDeptVO();

            for (SysUserVO user : userList) {
                try {
                    // 通过手机号验证是否存在这个用户
                    SysUser old = this.selectUserByUserNo(user.getUserNo());
                    transactionTemplate.execute(status -> {

                        user.setDeptId(deptList.stream().filter(d -> Objects.equals(d.getDeptName(), user.getDeptName())).findFirst().orElse(defaultDept).getDeptId());

                        if (StringUtils.isNull(old)) {
                            BeanValidators.validateWithException(validator, ValidGroup.Create.class);
                            deptService.checkDeptDataScope(user.getDeptId());
                            String password = configService.selectConfigByKey("sys.user.initPassword");
                            user.setPassword(SecurityUtils.encryptPassword(password));
                            user.setCreateBy(operator.getNickName());
                            int insert = userMapper.insertUser(user);
                            ServiceUtil.assertion(insert != 1, "新增用户失败");
                            return insert;
                        }
                        else if (isUpdateSupport) {
                            BeanValidators.validateWithException(validator, ValidGroup.Update.class);
                            checkUserAllowed(old);
                            user.setUserId(old.getUserId());
                            user.setUpdateBy(operator.getNickName());
                            int update = userMapper.updateUser(user);
                            ServiceUtil.assertion(update != 1, "修改用户失败");
                            return update;
                        }
                        else {
                            String msg = "用户工号为 " + user.getUserNo() + " 的数据已存在";
                            return 0;
                        }
                    });
                } catch (Exception e) {
                    String msg = "用户工号为 " + user.getUserNo() + " 的数据导入失败:" + e.getMessage();
                    log.error(msg, e);
                }
            }

        }, 0, TimeUnit.SECONDS);


        int rows = userList.size();
        if (rows > 0) {
            clearCache();
        }
        return rows;
    }

    private SysUser selectUserByUserNo(String userNo) {
        if (StringUtils.isBlank(userNo)) {
            return null;
        }
        SysUserQuery query = new SysUserQuery();
        query.setEqUserNo(userNo);
        return selectOne(query);
    }

    @Override
    public SysUser selectUserByPhonenumber(String phonenumber) {
        if (StringUtils.isBlank(phonenumber)) {
            return null;
        }
        SysUserQuery query = new SysUserQuery();
        query.setEqPhonenumber(phonenumber);
        return this.selectOne(query);
    }

    @Override
    public int selectCount(SysUserQuery query) {
        return userMapper.selectCount(query);
    }

    @Override
    public String bindWxByLoginCode(Long userId, String loginCode) {
//        if (userId == null || StringUtils.isBlank(loginCode)) {
//            log.error("绑定微信失败：参数错误 userId = {}, loginCode = {}", userId, loginCode);
//            return null;
//        }
//
//        // 获取微信openId
//        String openId = wxAuthService.getOpenId(loginCode);
//        ServiceUtil.assertion(StringUtils.isBlank(openId), "获取微信openId失败");
//
//        // 获取用户信息
//        SysUser user = selectUserById(userId);
//        ServiceUtil.assertion(user == null, "用户不存在");
//        ServiceUtil.assertion(StringUtils.hasText(user.getWxOpenId()), "当前用户已绑定微信，请解绑后重试");
//
//        // 操作绑定
//        Integer result = transactionTemplate.execute(status -> {
//            int update = userMapper.bindWxOpenId(user.getUserId(), openId);
//            ServiceUtil.assertion(update != 1, "绑定微信失败，当前用户信息已发生变化，请重试");
//            return update;
//        });
//
//        if (result != null && result == 1) {
//            return openId;
//        } else {
//            return null;
//        }
        return null;
    }

    @Override
    public SysUserVO selectByWxOpenId(String openId) {
        if (StringUtils.isBlank(openId)) {
            return null;
        }
        SysUserQuery query = new SysUserQuery();
        query.setEqWxOpenId(openId);
        return this.selectOne(query);
    }

    @Override
    public int unbindWx(Long userId) {
        return userMapper.unbindWxOpenId(userId);
    }

    @Override
    public List<SysUserVO> selectUserList(SysUserQuery query) {
        return userMapper.selectUserList(query);
    }

    @Override
    public List<SysUserNameVO> selectAllUserNameList() {
        List<SysUserNameVO> list = redisCache.getCacheObject(CacheConstants.USER_NAME_LIST);
        if (CollectionUtils.isEmptyElement(list)) {
            SysUserQuery query = new SysUserQuery();
            query.setExcludeUserId(1L);
            list = userMapper.selectNameList(query);
            if (CollectionUtils.isNotEmptyElement(list)) {
                redisCache.setCacheObject(CacheConstants.USER_NAME_LIST, list);
            }
        }
        return list;
    }

    @Override
    public List<SysUserVO> selectByIds(List<Long> ids) {
        if (CollectionUtils.isEmptyElement(ids)) {
            return Collections.emptyList();
        }
        SysUserQuery query = new SysUserQuery();
        query.setUserIds(ids);
        return userMapper.selectUserList(query);
    }

    private SysUserVO selectOne(SysUserQuery query) {
        PageHelper.startPage(1, 1);
        List<SysUserVO> list = userMapper.selectUserList(query);
        if (CollectionUtils.isEmpty(list)) {
            return null;
        }
        return list.get(0);
    }

    @Override
    public List<SysUserVO> selectUserListByNickNames(List<String> nickNames) {
        if (CollectionUtils.isEmptyElement(nickNames)) {
            return Collections.emptyList();
        }

        SysUserQuery query = new SysUserQuery();
        query.setNickNames(nickNames);
        return userMapper.selectUserList(query);
    }

    @Override
    public List<Long> selectUserIdsByDeptIds(List<Long> deptIds) {
        if (CollectionUtils.isEmptyElement(deptIds)) {
            return Collections.emptyList();
        }
        SysUserQuery query = new SysUserQuery();
        query.setDeptIds(deptIds);
        return userMapper.selectUserIds(query);
    }
}
